/***************************************************
 *
 * viewer.c
 *
 * Author: John Heathco <john@heathcosoft.com>
 *
 * Viewing userland application for the empeg -
 * runs on hijack and binds as 'Viewer' to the menu.
 * Allows to go thru directories and display files.
 *
 * Installation:
 * -------------
 * 
 * Put the executable (viewer) onto your player
 * via ftp (must have hijack) somewhere.
 * 
 * Use preinit (by Bitt Faulk) to have the userland
 * app start on boot.
 *
 * Controls:
 * ---------
 * 
 * WHILE BROWSING FOLDERS/FILES
 * -Knob-right/vol-down goes to next file/folder
 * -Knob-left/vol-up goes to prev file/folder
 * -Knob-press/menu/bottom button goes into folder/shows file
 * -Top-button/cancel goes up directory and quits at root folder
 *
 * WHILE SHOWING FILE
 * -Knob-right/vol-down goes to next line in file
 * -Knob-left/vol-up goes to previous line in file
 * -Knob-press/menu/bottom button goes to top of file
 * -Top-button/cnacel goes back to listing
 * 
 * -------------------------------------------------
 * Feel free to use this source file and change it.
 * Updates are encouraged because I won't be working
 * on it anymore... too busy with school.
 * -------------------------------------------------
 *
 ***************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>	
#include <sys/fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <dirent.h>
#include "hijack.h"

// Maximum number of file entries to parse per directory
#define MAX_ENTRIES				50

/***************************************************
 *
 * Updates the current file display position
 *
 ***************************************************/
int update_pos( char* buffer, int cur_pos, int next_line )
{
    int temp;
    int buffer_length;  
    int done;

    buffer_length = strlen( buffer );

    if( next_line < 0 )
    {
	for( temp = (cur_pos - 2);
		(temp >= 0) && (buffer[temp] != '\n');
		temp-- );

	if( ++temp < 0 )
	    temp = 0;

	cur_pos = temp;
    }
    else
    {
	for( temp = cur_pos;
		(temp < buffer_length) && (buffer[temp] != '\n');
		temp++ );

	if( ++temp > (buffer_length - 1) )
	    temp = (buffer_length - 1);

	cur_pos = temp;
    }

    return cur_pos;
}


/***************************************************
 *
 * Displays selected file
 *
 ***************************************************/
void show_file( int fd, char* filename )
{
    int file_id;
    char buffer[10241];
    int num_read;
    int cur_pos;
    int button_press;

    file_id = open( filename, O_RDONLY );
    if( file_id != -1 )
    {
	num_read = read( file_id, buffer, 10240 );
	buffer[num_read] = '\0';
	close( file_id );

	cur_pos = 0;
	button_press = -1;

	while(  (button_press != IR_RIO_CANCEL_PRESSED) &&
		(button_press != IR_TOP_BUTTON_PRESSED) )
	{
	    ioctl( fd, EMPEG_HIJACK_DISPCLEAR, 0 );
	    ioctl( fd, EMPEG_HIJACK_DISPTEXT, buffer + cur_pos );
	    ioctl( fd, EMPEG_HIJACK_WAITBUTTONS, &button_press );

	    if( (button_press == IR_KNOB_RIGHT) ||
		(button_press == IR_RIO_VOLMINUS_PRESSED) )
		cur_pos = update_pos( buffer, cur_pos, 1 );      

	    else if( (button_press == IR_KNOB_LEFT) ||
		     (button_press == IR_RIO_VOLPLUS_PRESSED) )
		cur_pos = update_pos( buffer, cur_pos, -1 );

	    else if( (button_press == IR_RIO_MENU_PRESSED) ||
		     (button_press == IR_BOTTOM_BUTTON_PRESSED) ||
		     (button_press == IR_KNOB_PRESSED) )
		cur_pos = 0;
	}
    }
}

/***************************************************
 *
 * Viewer main function
 *
 ***************************************************/
void show_viewer( int fd )
{
    char cur_path[PATH_MAX + 1];
    int button_press;
    struct dirent* cur_ent;
    struct dirent entries[MAX_ENTRIES];
    struct stat filestat;
    char full_name[PATH_MAX + 1];
    int i;
    int done;
    int cur_pos;
    int num_entries;
    char display[NAME_MAX * 4];

    DIR* dir;

    strcpy( cur_path, "/" );

    button_press = -1;
    while( (button_press != IR_RIO_CANCEL_PRESSED) &&
	   (button_press != IR_TOP_BUTTON_PRESSED) )
    {
	if( dir = opendir( cur_path ) )
	{
	    num_entries = 0;
	    ioctl( fd, EMPEG_HIJACK_DISPCLEAR, 0 );
	    ioctl( fd, EMPEG_HIJACK_DISPTEXT, "Reading folder listing..." );      

	    for( i = 0; i < MAX_ENTRIES; i++ )
	    {
		cur_ent = readdir( dir );
		if( cur_ent )
		{
		    strcpy( full_name, cur_path );
		    strcat( full_name, cur_ent->d_name );

		    if( (stat( full_name, &filestat ) == 0) &&
			(S_ISDIR( filestat.st_mode ) ||
			 S_ISREG( filestat.st_mode )) )
		    {
			if( strcmp( cur_ent->d_name, "." ) &&
			    strcmp( cur_ent->d_name, ".." ) )
			{
			    memcpy( &entries[num_entries], cur_ent,
				    sizeof( struct dirent ) );
			    num_entries++;
			}
		    }
		}
		else
		    break;
	    }

	    done = 0;
	    cur_pos = 0;

	    while( !done )
	    {
		if( strcmp( cur_path, "/" ) )
		{
		    for( i = (strlen( cur_path ) - 2);
			 cur_path[i] != '/';
			 i-- );
		    sprintf( display, "--- %s\n", cur_path + i + 1 );
		}
		else
		    strcpy( display, "--- (root)\n" );

		if( num_entries == 0 )
		{
		    strcat( display, "Empty folder!\n" );
		}
		else
		{
		    for( i = cur_pos;
			 (i < num_entries) && (i <= (cur_pos + 2));
			 i++ )
		    {
			strcpy( full_name, cur_path );
			strcat( full_name, entries[i].d_name );
			stat( full_name, &filestat );

			if( S_ISDIR( filestat.st_mode ) )
			    sprintf( strlen( display ) + display,
				     "->%s\n", entries[i].d_name );
			else 
			    sprintf( strlen( display ) + display,
				     "%s\n", entries[i].d_name );
		    }
		}       

		ioctl( fd, EMPEG_HIJACK_DISPCLEAR, 0 );
		ioctl( fd, EMPEG_HIJACK_DISPTEXT, display );
		ioctl( fd, EMPEG_HIJACK_WAITBUTTONS, &button_press );  

		if( (button_press == IR_KNOB_RIGHT) ||
		    (button_press == IR_RIO_VOLMINUS_PRESSED) )
		{
		    if( ++cur_pos > (num_entries - 1) )
			cur_pos = num_entries - 1;
		}
		else if( (button_press == IR_KNOB_LEFT) ||
			 (button_press == IR_RIO_VOLPLUS_PRESSED) )
		{
		    if( --cur_pos < 0 )
			cur_pos = 0;
		}
		else if( ((button_press == IR_BOTTOM_BUTTON_PRESSED) ||
			  (button_press == IR_RIO_MENU_PRESSED) ||
			  (button_press == IR_KNOB_PRESSED)) &&
			 (num_entries > 0) )
		{
		    if( num_entries > 0 )
		    {
			strcpy( full_name, cur_path );
			strcat( full_name, entries[cur_pos].d_name );
			stat( full_name, &filestat );

			if( S_ISDIR( filestat.st_mode ) )
			{
			    strcat( full_name, "/" );
			    strcpy( cur_path, full_name );

			    done = 1;
			}
			else
			{
			    strcpy( full_name, cur_path );
			    strcat( full_name, entries[cur_pos].d_name );

			    show_file( fd, full_name );
			}
		    }
		}         
		else if( (button_press == IR_TOP_BUTTON_PRESSED) ||
			 (button_press == IR_RIO_CANCEL_PRESSED) )
		{
		    if( strcmp( cur_path, "/" ) )
		    {
			for( i = (strlen( cur_path ) - 2);
			     cur_path[i] != '/';
			     i-- );

			cur_path[i + 1] = '\0';

			button_press = -1;           
		    }            
		    done = 1;
		}
	    }
	}
	else
	{
	    ioctl( fd, EMPEG_HIJACK_DISPCLEAR, 0 );
	    ioctl( fd, EMPEG_HIJACK_DISPTEXT, "Unable to open directory!" );
	}
    }
}

/***************************************************
 *
 * Entrance function
 *
 ***************************************************/
int main( int nargs, char** args )
{
    int fd;
    int rc;
    int menu_index;
    int i;
    int done; // Loop boolean

    // Buttons to be hijacked
    unsigned long buttons[] = { 14,
	IR_RIO_NEXTTRACK_PRESSED,
	IR_RIO_PREVTRACK_PRESSED,
	IR_RIO_MENU_PRESSED,
	IR_RIO_CANCEL_PRESSED,
	IR_TOP_BUTTON_PRESSED,
	IR_RIO_VOLMINUS_PRESSED,
	IR_RIO_VOLPLUS_PRESSED, 
	IR_RIGHT_BUTTON_PRESSED,
	IR_LEFT_BUTTON_PRESSED,
	IR_KNOB_RIGHT,
	IR_KNOB_LEFT,
	IR_KNOB_PRESSED,
	IR_RIO_PLAY_PRESSED
    };

    unsigned char screen[EMPEG_SCREEN_BYTES] = { 0 };
    hijack_geom_t fullscreen = { 0, EMPEG_SCREEN_ROWS - 1,
				 0, EMPEG_SCREEN_COLS - 1 };

#if USING_HIJACK_MENU
    char *menu[] = { "Viewer", NULL }; // Menu name

    if( fork( ) != 0 ) // Fork off the process, if parent, return
	return 0;
    else
    { 
#endif

	fd = open( "/dev/display", O_RDWR );

	if( !fd )
	{
	    perror( "Couldn't connect to display!\n" );
	    return 0;
	}

#if USING_HIJACK_MENU
    // Loop executed while waiting to be chosen from the menu...
    while( 1 )
    {
	rc = ioctl( fd, EMPEG_HIJACK_WAITMENU, menu ); // Wait for menu..
	if( rc < 0 )
	{
	    perror( "Couldn't connect to Hijack menu!\n" );
	    close( fd );

	    return 0;
	}
	else
	    menu_index = rc;
#endif

	// Bind buttons
	rc = ioctl( fd, EMPEG_HIJACK_BINDBUTTONS, buttons );

	// Grab screen
	rc = ioctl( fd, EMPEG_HIJACK_SETGEOM, &fullscreen );

	show_viewer( fd );

	// Left our program... unbind menu
	rc = ioctl( fd, EMPEG_HIJACK_UNBINDBUTTONS, NULL );
#if USING_HIJACK_MENU
    }      
#endif

	// Close the display
	close( fd );

#if USING_HIJACK_MENU
    }
#else
    return 0;
#endif
}
